/* AFC - The  Advanced Foundation Classes
 * Copyright (C) 2000/2004  Fabio Rotondo - Andrea Galimberti
 *  
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "scrolledwindow.h"

static const char class_name[] = "EGUI Widget ScrolledWindow";

static int afc_easygui_internal_scrolledwindow_get_template ( DynamicClass * dyn );
static int afc_easygui_internal_cb_scrolledwindow_start ( DynamicClass * dyn );
static int afc_easygui_internal_cb_scrolledwindow_end ( DynamicClass * dyn );

static int afc_easygui_scrolledwindow_method_add ( DynamicClass * dyn );

DynamicClass * afc_easygui_internal_scrolledwindow_new_instance ()
{
	DynamicClass * dyn = NULL;

	if ( ( dyn = afc_dynamic_class_new () ) == NULL ) {
		AFC_LOG_FAST_INFO ( AFC_ERR_NO_MEMORY, "dynamic class: scrolledwindow" );
		return ( NULL );
	}

	afc_dynamic_class_add_method ( dyn, "get_template", NULL,  afc_easygui_internal_scrolledwindow_get_template );
	afc_dynamic_class_add_method ( dyn, "open_callback", NULL,  afc_easygui_internal_cb_scrolledwindow_start );
	afc_dynamic_class_add_method ( dyn, "close_callback", NULL,  afc_easygui_internal_cb_scrolledwindow_end );
	afc_dynamic_class_add_method ( dyn, "enabled",	NULL, 	afc_easygui_widget_method_enabled );

	afc_dynamic_class_add_method ( dyn, "add", NULL,  afc_easygui_scrolledwindow_method_add );

	return ( dyn );
}

int afc_easygui_internal_scrolledwindow_del_instance ( DynamicClass * dyn )
{
	return ( afc_easygui_widget_standard_free ( dyn ) );
	/*
	EasyGUIWidgetData * ewd;

	if ( dyn == NULL ) return ( AFC_ERR_INVALID_POINTER );

	WIDGET_FREE_EWIDGET ( dyn, ewd );

	afc_dynamic_class_delete ( dyn );

	return ( AFC_ERR_NO_ERROR );
	*/
}

static int afc_easygui_internal_scrolledwindow_get_template ( DynamicClass * dyn )
{
	dyn->result = "HSCROLL VSCROLL";
	return ( AFC_ERR_NO_ERROR );
}

static int afc_easygui_internal_cb_scrolledwindow_start ( DynamicClass * dyn )
{
	EasyGUI * egui = dyn->info;
	EasyGUIContext * context = egui->curr_context;
	GtkWidget * w;
	char * hpol_str = NULL;
	char * vpol_str = NULL;
	int afc_res = AFC_ERR_NO_ERROR;
	GtkPolicyType hpol = GTK_POLICY_AUTOMATIC;
	GtkPolicyType vpol = GTK_POLICY_AUTOMATIC;
	DynamicClass * dc;

	if ( context->curr_container == NULL ) return ( AFC_EASYGUI_ERR_NO_CONTAINER );
	
	afc_easygui_widget_parse_default_params ( egui );

	hpol_str = afc_cmd_parser_arg_get_by_name ( egui->parser, "HSCROLL" );
	vpol_str = afc_cmd_parser_arg_get_by_name ( egui->parser, "VSCROLL" );

	if ( ( dc = afc_dynamic_class_master_new_instance ( egui->commands, "scrolledwindow" ) ) == NULL )
		return ( AFC_LOG_FAST_INFO ( AFC_ERR_NO_MEMORY, "dynclass instance" ) );

	WIDGET_SET_CONTEXT ( dc, context );

	w = gtk_scrolled_window_new ( NULL, NULL );

	if ( ( afc_res = afc_easygui_context_add_container ( context, w, dc, AFC_EASYGUI_CONTAINER_TYPE_SCROLLEDW, AFC_EASYGUI_CONTAINER_HOMOGENEOUS_NONE ) ) == AFC_ERR_NO_ERROR ) {
		if ( hpol_str != NULL ) hpol = ( strcasecmp ( hpol_str, "ALWAYS" ) == 0 ) ? GTK_POLICY_ALWAYS : GTK_POLICY_AUTOMATIC;
		if ( vpol_str != NULL ) vpol = ( strcasecmp ( vpol_str, "ALWAYS" ) == 0 ) ? GTK_POLICY_ALWAYS : GTK_POLICY_AUTOMATIC;
		gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW ( w ), hpol, vpol );
	}

	return ( afc_res );
}

static int afc_easygui_internal_cb_scrolledwindow_end ( DynamicClass * dyn )
{
	EasyGUI * egui = dyn->info;

	return ( afc_easygui_context_close_container ( egui->curr_context ) );
}

// METHODS
static int afc_easygui_scrolledwindow_method_add ( DynamicClass * dyn )
{
	GtkWidget * w;
	EasyGUIWidgetData * ewidget;

	w = afc_array_master_first ( dyn->args );
	WIDGET_GET_EWIDGET ( dyn, ewidget );	

	gtk_scrolled_window_add_with_viewport ( GTK_SCROLLED_WINDOW ( ewidget->gtk_handle ), w );

	return ( AFC_ERR_NO_ERROR );
}
