/* AFC - The  Advanced Foundation Classes
 * Copyright (C) 2000/2004  Fabio Rotondo - Andrea Galimberti
 *  
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "toolbar.h"

static const char class_name [] = "EGUI Widget Toolbar";

static int afc_easygui_internal_toolbar_get_template ( DynamicClass * dyn );
static int afc_easygui_internal_cb_toolbar ( DynamicClass * dyn );
static int afc_easygui_internal_cb_toolbar_end ( DynamicClass * dyn );

static int afc_easygui_toolbar_method_add ( DynamicClass * dyn );

DynamicClass * afc_easygui_internal_toolbar_new_instance ()
{
	DynamicClass * dyn = NULL;

	if ( ( dyn = afc_dynamic_class_new () ) == NULL ) {
		AFC_LOG_FAST_INFO ( AFC_ERR_NO_MEMORY, "dynamic class: toolbar" );
		return ( NULL );
	}

	afc_dynamic_class_add_method ( dyn, "get_template", NULL,  afc_easygui_internal_toolbar_get_template );
	afc_dynamic_class_add_method ( dyn, "open_callback", NULL,  afc_easygui_internal_cb_toolbar );
	afc_dynamic_class_add_method ( dyn, "close_callback", NULL,  afc_easygui_internal_cb_toolbar_end );
	afc_dynamic_class_add_method ( dyn, "enabled",	NULL, 	afc_easygui_widget_method_enabled );

	afc_dynamic_class_add_method ( dyn, "add", NULL,  afc_easygui_toolbar_method_add );

	return ( dyn );
}

int afc_easygui_internal_toolbar_del_instance ( DynamicClass * dyn )
{
	return ( afc_easygui_widget_standard_free ( dyn ) );
}

static int afc_easygui_internal_toolbar_get_template ( DynamicClass * dyn )
{
	dyn->result = "";
	return ( AFC_ERR_NO_ERROR );
}

static int afc_easygui_internal_cb_toolbar ( DynamicClass * dyn )
{
	EasyGUI * egui = dyn->info;
	EasyGUIContext * context = egui->curr_context;
	GtkWidget * w;
	DynamicClass * dc;

	if ( context->curr_container == NULL ) return ( AFC_EASYGUI_ERR_NO_CONTAINER );

	afc_easygui_widget_parse_default_params ( egui );

	if ( ( dc = afc_dynamic_class_master_new_instance ( egui->commands, "toolbar" ) ) == NULL )
		return ( AFC_LOG_FAST_INFO ( AFC_ERR_NO_MEMORY, "dynclass instance" ) );

	WIDGET_SET_CONTEXT ( dc, context );

	if ( ( w = gtk_toolbar_new () ) == NULL ) return ( AFC_LOG_FAST_INFO ( AFC_ERR_NO_MEMORY, "toolbar" ) );

	return ( afc_easygui_context_add_container ( context, w, dc, AFC_EASYGUI_CONTAINER_TYPE_TOOLBAR, AFC_EASYGUI_CONTAINER_HOMOGENEOUS_NONE ) );
}

static int afc_easygui_internal_cb_toolbar_end ( DynamicClass * dyn )
{
	EasyGUI * egui = dyn->info;

	return ( afc_easygui_context_close_container ( egui->curr_context ) );
}

// METHODS
static int afc_easygui_toolbar_method_add ( DynamicClass * dyn )
{
	GtkWidget * w;
	EasyGUIWidgetData * ewidget;

	w = afc_array_master_first ( dyn->args );
	WIDGET_GET_EWIDGET ( dyn, ewidget );	

	gtk_toolbar_append_element ( 	GTK_TOOLBAR ( ewidget->gtk_handle ),
					GTK_TOOLBAR_CHILD_WIDGET, w,
					NULL, NULL,
					NULL, NULL,
					NULL, NULL );	

	return ( AFC_ERR_NO_ERROR );
}
