/* AFC - The  Advanced Foundation Classes
 * Copyright (C) 2000/2004  Fabio Rotondo - Andrea Galimberti
 *  
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef AFC_MEM_TRACKER_H
#define AFC_MEM_TRACKER_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#ifndef AFC_BASE_H
#include "base.h"
#endif

#include "string.h"
#include "exceptions.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* AFC afc_mem_tracker Magic Number */
#define AFC_MEM_TRACKER_MAGIC ( 'A' << 24 | 'R' << 16 | 'R' << 8 | 'A' )

/* AFC afc_mem_tracker Base value for constants */
#define AFC_MEM_TRACKER_BASE 0x8000


/* Errors for afc_mem_tracker */

#define AFC_MEM_TRACKER_DEFAULT_ITEMS       100

enum {	AFC_MEM_TRACKER_ADD_HERE = AFC_MEM_TRACKER_BASE + 1,
	AFC_MEM_TRACKER_ADD_TAIL,
	AFC_MEM_TRACKER_ADD_HEAD
};

#define AFC_MEM_TRACKER_SORT_ELEMENT(kind,varname)	( kind ) ( * ( kind * ) varname )

struct mem_tracker_internal_data
{
	unsigned long int hash_value;
	void * data;
	size_t size;
};

typedef struct mem_tracker_internal_data MemTData;

struct afc_mem_tracker 
{
  	unsigned long magic;

	void * * mem;

  	unsigned long int max_items;
  	unsigned long int current_pos;
  	unsigned long int num_items;

	short is_sorted;
	short show_mallocs;
	short show_frees;

	int ( *func_clear ) ( void * );
};

typedef struct afc_mem_tracker MemTracker;

#define afc_mem_tracker_delete(mt)	if ( mt ) { _afc_mem_tracker_delete ( mt ); mt = NULL; }

struct afc_mem_tracker * afc_mem_tracker_new ( void );
int    _afc_mem_tracker_delete   ( struct afc_mem_tracker * ) ;
int    afc_mem_tracker_clear    ( struct afc_mem_tracker * ) ;
void * afc_mem_tracker_malloc   ( MemTracker *, size_t size );
void   afc_mem_tracker_free     ( MemTracker *, void * mem );

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
